@@ -51,8 +51,6 @@ package com.openexchange.pns.impl;
 
 import static com.openexchange.java.Autoboxing.Coll2i;
 import static com.openexchange.java.Autoboxing.I;
-import static com.openexchange.java.Autoboxing.L;
-import static com.openexchange.logging.LogUtility.toStringObjectFor;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
@@ -61,6 +59,7 @@ import java.util.LinkedHashMap;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
+import java.util.Map.Entry;
 import java.util.concurrent.atomic.AtomicLong;
 import java.util.concurrent.locks.Lock;
 import java.util.concurrent.locks.ReentrantLock;
@@ -223,7 +222,7 @@ public class PushNotificationServiceImpl implements PushNotificationService {
 
     private final Processor processor;
 
-    private final UnsynchronizedBufferingQueue<PushNotification> submittedNotifcations; // Accessed synchronized
+    private final UnsynchronizedBufferingQueue<PushNotification> scheduledNotifcations; // Accessed synchronized
     private ScheduledTimerTask scheduledTimerTask; // Accessed synchronized
     private boolean stopped; // Accessed synchronized
 
@@ -246,7 +245,7 @@ public class PushNotificationServiceImpl implements PushNotificationService {
         this.configService = configService;
         this.timerService = timerService;
         this.transportRegistry = transportRegistry;
-        submittedNotifcations = new UnsynchronizedBufferingQueue<>(delayDuration(configService), maxDelayDuration(configService));
+        scheduledNotifcations = new UnsynchronizedBufferingQueue<>(delayDuration(configService), maxDelayDuration(configService));
         numOfProcessedNotifications = new AtomicLong(0);
         numOfSubmittedNotifications = new AtomicLong(0);
     }
@@ -260,7 +259,7 @@ public class PushNotificationServiceImpl implements PushNotificationService {
                 return 0L;
             }
 
-            return submittedNotifcations.size();
+            return scheduledNotifcations.size();
         } finally {
             lock.unlock();
         }
@@ -290,7 +289,7 @@ public class PushNotificationServiceImpl implements PushNotificationService {
             }
 
             if (handleRemaining) {
-                for (PushNotification notification : submittedNotifcations) {
+                for (PushNotification notification : scheduledNotifcations) {
                     try {
                         new NotificationsHandler(notification).run();
                     } catch (Exception e) {
@@ -309,7 +308,7 @@ public class PushNotificationServiceImpl implements PushNotificationService {
                 processor.stop();
             }
 
-            submittedNotifcations.clear();
+            scheduledNotifcations.clear();
             cancelTimerTask();
             stopped = true;
         } finally {
@@ -323,25 +322,33 @@ public class PushNotificationServiceImpl implements PushNotificationService {
      * May only be accessed when holding lock.
      */
     protected void cancelTimerTask() {
-        ScheduledTimerTask scheduledTimerTask = this.scheduledTimerTask;
         if (null != scheduledTimerTask) {
-            this.scheduledTimerTask = null;
             scheduledTimerTask.cancel();
+            scheduledTimerTask = null;
         }
     }
 
+    @Override
+    public void handle(PushNotification notification) throws OXException {
+        handle(notification, PushPriority.HIGH);
+    }
+
+    @Override
+    public void handle(PushNotification notification, PushPriority priority) throws OXException {
+        handle(Collections.singletonList(notification), priority);
+    }
+
     @Override
     public void handle(Collection<PushNotification> notifications, PushPriority priority) throws OXException {
         if (null == notifications || notifications.isEmpty()) {
             return;
         }
 
-        long delayDuration = delayDuration(configService, priority); // Default 1 second
-        long maxDelayDuration = maxDelayDuration(configService, priority); // Default 10 seconds
+        long delayDuration = delayDuration(configService, priority);
+        long maxDelayDuration = maxDelayDuration(configService, priority);
         lock.lock();
         try {
             if (stopped) {
-                LOGGER.debug("Push notification service stopped. Aborting enqueueing of notifications...");
                 return;
             }
 
@@ -361,10 +368,9 @@ public class PushNotificationServiceImpl implements PushNotificationService {
                         checkNotifications();
                     }
                 };
-                long delay = timerFrequency(configService); // Default 0,5 second
-                long initialDelay = delayDuration + 500L; // Default 1 second + 0,5 second
+                long initialDelay = delayDuration(configService);
+                long delay = timerFrequency(configService);
                 scheduledTimerTask = timerService.scheduleWithFixedDelay(task, initialDelay, delay);
-                LOGGER.debug("Initiated polling timer task with initial delay of {}msec and periodic delay of {}msec", L(initialDelay), L(delay));
             }
 
             // Add rest to queue
@@ -377,14 +383,14 @@ public class PushNotificationServiceImpl implements PushNotificationService {
     }
 
     private void addToQueue(PushNotification notification, long delayDuration, long maxDelayDuration) {
-        boolean added = submittedNotifcations.offerIfAbsentElseReset(notification, delayDuration, maxDelayDuration);
+        boolean added = scheduledNotifcations.offerIfAbsentElseReset(notification, delayDuration, maxDelayDuration);
         if (added) {
             if (numOfSubmittedNotifications.incrementAndGet() < 0L) {
                 numOfSubmittedNotifications.set(0L);
             }
-            LOGGER.debug("Enqueued notification \"{}\" for user {} in context {} with delay of {}msec ({}msec max. delay)", notification.getTopic(), I(notification.getUserId()), I(notification.getContextId()), L(delayDuration), L(maxDelayDuration));
+            LOGGER.debug("Scheduled notification \"{}\" for user {} in context {}", notification.getTopic(), I(notification.getUserId()), I(notification.getContextId()));
         } else {
-            LOGGER.debug("Reset & re-enqueued notification \"{}\" for user {} in context {} with delay of {}msec ({}msec max. delay)", notification.getTopic(), I(notification.getUserId()), I(notification.getContextId()), L(delayDuration), L(maxDelayDuration));
+            LOGGER.debug("Reset & re-scheduled notification \"{}\" for user {} in context {}", notification.getTopic(), I(notification.getUserId()), I(notification.getContextId()));
         }
     }
 
@@ -392,82 +398,55 @@ public class PushNotificationServiceImpl implements PushNotificationService {
      * Checks for an available notifications.
      */
     protected void checkNotifications() {
-        List<PushNotification> polled;
-        boolean onlyOne;
+        List<PushNotification> polled = null;
+        int numAdded = 0;
 
         // Leave this mutually exclusive section as fast as possible...
         lock.lock();
         try {
             if (stopped) {
-                LOGGER.debug("Push notification service stopped. Aborting polling of queue...");
                 return;
             }
 
-            PushNotification first = submittedNotifcations.poll();
-            if (null == first) {
+            PushNotification notification = scheduledNotifcations.poll();
+            if (null == notification) {
                 // Queue has no notification with an expired delay
-                LOGGER.debug("Queue has no notification with an expired delay and holds {} buffered notifications", I(submittedNotifcations.size()));
                 return;
             }
 
             // Check for more
-            PushNotification notification = submittedNotifcations.poll();
-            if (notification != null) {
-                polled = new LinkedList<>();
-                polled.add(first);
-                LOGGER.debug("Polled notification \"{}\" for user {} in context {} from queue", first.getTopic(), I(first.getUserId()), I(first.getContextId()));
-                do {
-                    polled.add(notification);
-                    LOGGER.debug("Polled notification \"{}\" for user {} in context {} from queue", notification.getTopic(), I(notification.getUserId()), I(notification.getContextId()));
-                    notification = submittedNotifcations.poll();
-                } while (null != notification);
-                onlyOne = false;
-            } else {
-                polled = Collections.singletonList(first);
-                LOGGER.debug("Polled notification \"{}\" for user {} in context {} from queue", first.getTopic(), I(first.getUserId()), I(first.getContextId()));
-                onlyOne = true;
-            }
+            polled = new LinkedList<>();
+            do {
+                polled.add(notification);
+                numAdded++;
+                notification = scheduledNotifcations.poll();
+            } while (null != notification);
 
-            if (submittedNotifcations.isEmpty()) {
+            if (scheduledNotifcations.isEmpty()) {
                 cancelTimerTask();
-                LOGGER.debug("Dropped polling timer task since queue is currently empty");
             }
         } finally {
             lock.unlock();
         }
 
-        if (onlyOne) {
+        if (1 == numAdded) {
             // There was only a single one available
             PushNotification first = polled.get(0);
             NotificationsHandler task = new NotificationsHandler(first);
-            if (tryExecuteTask(task, first.getUserId(), first.getContextId())) {
-                LOGGER.debug("Submitted delivery of polled notification with topic \"{}\" to user {} in context {}", first.getTopic(), I(first.getUserId()), I(first.getContextId()));
-            } else {
+            if (false == tryExecuteTask(task, first.getUserId(), first.getContextId())) {
                 // Processor rejected task execution. Perform with current thread.
                 task.run();
             }
         } else {
             // Handle them all
-            Map<ContextAndTopicKey, Map<Integer, List<PushNotification>>> polledNotificationsPerContextAndTopic = new LinkedHashMap<>();
+            Map<ContextAndTopicKey, Map<Integer, List<PushNotification>>> polledNotificationsPerUser = new LinkedHashMap<>();
             for (PushNotification notification : polled) {
-                put(notification, polledNotificationsPerContextAndTopic);
+                put(notification, polledNotificationsPerUser);
             }
-            for (Map.Entry<ContextAndTopicKey, Map<Integer, List<PushNotification>>> entry : polledNotificationsPerContextAndTopic.entrySet()) {
+            for (Entry<ContextAndTopicKey, Map<Integer, List<PushNotification>>> entry : polledNotificationsPerUser.entrySet()) {
                 ContextAndTopicKey key = entry.getKey();
-                Map<Integer, List<PushNotification>> notificationsPerUser = entry.getValue();
-                NotificationsHandler task = new NotificationsHandler(key.contextId, key.topic, notificationsPerUser);
-                if (tryExecuteTask(task, -1, key.contextId)) {
-                    if (LOGGER.isDebugEnabled()) {
-                        int[] userIds = Coll2i(notificationsPerUser.keySet());
-                        Object sUserIds = userIds.length <= 10 ? toStringObjectFor(userIds) : null;
-                        userIds = null;
-                        if (sUserIds == null) {
-                            LOGGER.debug("Submitted delivery of polled notifications with topic \"{}\" to users in context {}", key.topic, I(key.contextId));
-                        } else {
-                            LOGGER.debug("Submitted delivery of polled notifications with topic \"{}\" to users {} in context {}", key.topic, sUserIds, I(key.contextId));
-                        }
-                    }
-                } else {
+                NotificationsHandler task = new NotificationsHandler(key.contextId, key.topic, entry.getValue(), numAdded);
+                if (false == tryExecuteTask(task, -1, key.contextId)) {
                     // Processor rejected task execution. Perform with current thread.
                     task.run();
                 }
@@ -486,7 +465,7 @@ public class PushNotificationServiceImpl implements PushNotificationService {
             m = new HashMap<Integer, List<PushNotification>>();
             polledNotificationsPerUser.put(key, m);
         }
-        com.openexchange.tools.arrays.Collections.put(m, I(notification.getUserId()), notification);
+        com.openexchange.tools.arrays.Collections.put(m,  I(notification.getUserId()), notification);
     }
 
     /**
@@ -495,70 +474,51 @@ public class PushNotificationServiceImpl implements PushNotificationService {
      * @param contextId The context identifier (for all notifications)
      * @param topic The topic (for all notifications)
      * @param notificationsPerUser The notifications per user identifier
+     * @param numOfNotifications The number of notifications provided by given iterator
      * @throws OXException If handling fails
      */
-    protected void doHandle(int contextId, String topic, Map<Integer, List<PushNotification>> notificationsPerUser) throws OXException {
-        // Determine available user identifiers from "notifications per user" map
-        int[] userIds = Coll2i(notificationsPerUser.keySet());
-        Object sUserIds = userIds.length <= 10 && LOGGER.isDebugEnabled() ? toStringObjectFor(userIds) : null;
-        if (sUserIds == null) {
-            LOGGER.debug("Delivering notifications for topic \"{}\" to users in context {}", topic, I(contextId));
-        } else {
-            LOGGER.debug("Delivering notifications for topic \"{}\" to users {} in context {}", topic, sUserIds, I(contextId));
-        }
-
+    protected void doHandle(int contextId, String topic, Map<Integer, List<PushNotification>> notificationsPerUser, int numOfNotifications) throws OXException {
         // Query appropriate hits
+        int[] userIds = Coll2i(notificationsPerUser.keySet());
         Hits hits = subscriptionRegistry.getInterestedSubscriptions(userIds, contextId, topic);
         if (null == hits || hits.isEmpty()) {
-            if (sUserIds == null) {
-                LOGGER.debug("No subscriptions of interest for topic \"{}\" for users in context {}", topic, I(contextId));
-            } else {
-                LOGGER.debug("No subscriptions of interest for topic \"{}\" for users {} in context {}", topic, sUserIds, I(contextId));
-            }
-            addNumOfProcessedNotifications(getNumberOfNotificationsFrom(notificationsPerUser));
+            LOGGER.debug("No subscriptions of interest for topic \"{}\" for users {} in context {}", topic, userIds, I(contextId));
+            addNumOfProcessedNotifications(numOfNotifications);
             return;
         }
 
-        // Process by hits
-        if (sUserIds == null) {
-            LOGGER.debug("Determined subscriptions of interest for topic \"{}\" for users in context {}", topic, I(contextId));
-        } else {
-            LOGGER.debug("Determined subscriptions of interest for topic \"{}\" for users {} in context {}", topic, sUserIds, I(contextId));
-        }
-        for (Map.Entry<PushNotificationTransport, List<Hit>> entry : getHitsPerTransport(hits).entrySet()) {
-            // Get push matches for each notification
+        for (Entry<PushNotificationTransport, List<Hit>> entry : getHitsPerTransport(hits).entrySet()) {
+            /*
+             * get push matches for each notification
+             */
             PushNotificationTransport transport = entry.getKey();
             Map<PushNotification, List<PushMatch>> notifications = new HashMap<>();
             for (Hit hit : entry.getValue()) {
                 List<PushMatch> matches = hit.getMatches();
                 for (PushMatch match : matches) {
                     int userId = match.getUserId();
-                    if (isTransportAllowed(transport, topic, hit.getClient(), userId, contextId)) {
-                        List<PushNotification> notificationsForUser = notificationsPerUser.get(I(match.getUserId()));
-                        for (PushNotification notification : notificationsForUser) {
-                            if (matchesSourceToken(match, notification.getSourceToken())) {
-                                LOGGER.debug("Skipping push match for source token {}", notification.getSourceToken());
-                            } else {
-                                com.openexchange.tools.arrays.Collections.put(notifications, notification, match);
-                            }
-                        }
-                    } else {
-                        LOGGER.info("Transport '{}' not enabled for client '{}' to publish notification \"{}\" to user {} in context {}",
+                    if (false == isTransportAllowed(transport, topic, hit.getClient(), userId, contextId)) {
+                        LOGGER.info("Transport '{}' not enabled for client '{}' to publish notification \"{}\" from user {} in context {}",
                             transport.getId(), hit.getClient(), topic, I(userId), I(contextId));
+                        continue;
+                    }
+                    List<PushNotification> notificationsForUser = notificationsPerUser.get(I(match.getUserId()));
+                    for (PushNotification notification : notificationsForUser) {
+                        if (matchesSourceToken(match, notification.getSourceToken())) {
+                            LOGGER.debug("Skipping push match for source token {}", notification.getSourceToken());
+                            continue;
+                        }
+                        com.openexchange.tools.arrays.Collections.put(notifications, notification, match);
                     }
                 }
             }
-
-            // Hand over to associated transport
-            if (sUserIds == null) {
-                LOGGER.debug("Going to use \"{}\" transport to publish notification(s) with topic \"{}\" to users in context {}", transport.getId(), topic, I(contextId));
-            } else {
-                LOGGER.debug("Going to use \"{}\" transport to publish notification(s) with topic \"{}\" to users {} in context {}", transport.getId(), topic, sUserIds, I(contextId));
-            }
+            /*
+             * hand over to associated transport
+             */
             transport.transport(notifications);
         }
 
-        addNumOfProcessedNotifications(getNumberOfNotificationsFrom(notificationsPerUser));
+        addNumOfProcessedNotifications(numOfNotifications);
     }
 
     private Map<PushNotificationTransport, List<Hit>> getHitsPerTransport(Hits hits) throws OXException {
@@ -589,14 +549,6 @@ public class PushNotificationServiceImpl implements PushNotificationService {
         return null != sourceToken && null != match && sourceToken.equals(match.getToken());
     }
 
-    private int getNumberOfNotificationsFrom(Map<Integer, List<PushNotification>> notificationsPerUser) {
-        int count = 0;
-        for (List<PushNotification> notifications : notificationsPerUser.values()) {
-            count += notifications.size();
-        }
-        return count;
-    }
-
     private void addNumOfProcessedNotifications(int numOfNotifications) {
         if (numOfProcessedNotifications.addAndGet(numOfNotifications) < 0L) {
             numOfProcessedNotifications.set(0L);
@@ -608,30 +560,32 @@ public class PushNotificationServiceImpl implements PushNotificationService {
         private final Map<Integer, List<PushNotification>> notificationsPerUser;
         private final String topic;
         private final int contextId;
+        private final int numOfNotifications;
+
+        /**
+         * Initializes a new {@link PushNotificationServiceImpl.NotificationsHandler}.
+         */
+        NotificationsHandler(int contextId, String topic, Map<Integer, List<PushNotification>> notificationsPerUser, int numOfNotifications) {
+            super();
+            this.notificationsPerUser = notificationsPerUser;
+            this.topic = topic;
+            this.numOfNotifications = numOfNotifications;
+            this.contextId = contextId;
+        }
 
         /**
          * Initializes a new {@link PushNotificationServiceImpl.NotificationsHandler}.
          */
         NotificationsHandler(PushNotification notification) {
-            this(notification.getContextId(), notification.getTopic(), Collections.singletonMap(I(notification.getUserId()), Collections.singletonList(notification)));
-        }
-
-        /**
-         * Initializes a new {@link PushNotificationServiceImpl.NotificationsHandler}.
-         */
-        NotificationsHandler(int contextId, String topic, Map<Integer, List<PushNotification>> notificationsPerUser) {
-            super();
-            this.notificationsPerUser = notificationsPerUser;
-            this.topic = topic;
-            this.contextId = contextId;
+            this(notification.getContextId(), notification.getTopic(), Collections.singletonMap(I(notification.getUserId()), Collections.singletonList(notification)), 1);
         }
 
         @Override
         public void run() {
             try {
-                doHandle(contextId, topic, notificationsPerUser);
+                doHandle(contextId, topic, notificationsPerUser, numOfNotifications);
             } catch (Exception e) {
-                LOGGER.error("Failed to handle notification(s) with topic \"{}\" in context {}", topic, I(contextId), e);
+                LOGGER.error("Failed to handle notification(s) with topic {} in context {}", topic, I(contextId), e);
             }
         }
     }
